@@ -15,6 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import logging
 from synth_common import SynthCommon
 from mpd.track import MidiEventStream
 import winmidi
@@ -66,7 +67,7 @@ class WinSynth(SynthCommon):
             elif e[0] == midieventstream.SET_PATCH:
                 self.__driver.program_change(e[1], e[2])
             elif e[0] == midieventstream.BENDER:
-                print "ugh todo: seq_bender for play_with_drvmidi"
+                logging.debug("ugh todo: seq_bender for play_with_drvmidi")
                 #m.seq_bender(DEV, e[1], e[2])
             else:
                 pass
