@@ -20,7 +20,6 @@ import gu
 import abstract
 import lessonfile
 import mpd
-import utils
 import soundcard
 
 class Teacher(abstract.Teacher):
@@ -38,7 +37,7 @@ class Teacher(abstract.Teacher):
 class Gui(abstract.LessonbasedGui):
     def __init__(self, teacher, window):
         abstract.Gui.__init__(self, teacher, window)
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
+        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
         self.practise_box.pack_start(self.g_music_displayer)
         self.g_flashbar = gu.FlashBar()
         self.practise_box.pack_start(self.g_flashbar)
