@@ -17,7 +17,7 @@
 import gtk
 import gu
 from multipleintervalconfigwidget import MultipleIntervalConfigWidget
-import soundcard, mpd, mpd.musicdisplayer
+import mpd, mpd.musicdisplayer
 import abstract
 import statistics, statisticsviewer
 import utils
@@ -64,9 +64,7 @@ class Teacher(abstract.MelodicIntervalTeacher):
         if self.q_status == self.QSTATUS_NO:
             return
         assert self.m_question
-        soundcard.play_note(self.get_int('config/preferred_instrument'),
-                  4, self.m_tonika.semitone_pitch(),
-                  self.get_int('config/preferred_instrument_velocity'))
+        utils.play_note(4, self.m_tonika.semitone_pitch())
     def play_last_tone(self):
         if self.q_status == self.QSTATUS_NO:
             return
@@ -74,9 +72,7 @@ class Teacher(abstract.MelodicIntervalTeacher):
         t = self.m_tonika
         for i in self.m_question:
             t = t + i
-        soundcard.play_note(self.get_int('config/preferred_instrument'),
-                  4, t.semitone_pitch(),
-                  self.get_int('config/preferred_instrument_velocity'))
+        utils.play_note(4, t.semitone_pitch())
 
 
 class Gui(abstract.Gui):
@@ -85,7 +81,7 @@ class Gui(abstract.Gui):
         ################
         # practise_box #
         ################
-        self.g_score_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
+        self.g_score_displayer = mpd.musicdisplayer.MusicDisplayer()
         self.practise_box.pack_start(self.g_score_displayer)
         self.g_score_displayer.clear()
         b1 = gu.bHBox(self.practise_box, False)
