@@ -51,7 +51,7 @@ class TracebackWindow(gtk.Dialog):
         self.action_area.pack_start(self.g_close)
         self.g_close.connect('clicked', lambda w: self.hide())
     def do_report(self, *v):
-        yesno = gu.dialog_yesno(_("Automatic bug reports are often mostly useless because people omit their email address and add very little info about what happened. Fixing bugs is difficult if we cannot contact you and ask for more information.\n\nI would prefer if you open a web browser and report you bug to the bug tracker at http://bugs.solfege.org.\n\nThis will give your bug report higher priority and it will be fixed faster.\n\nAre you willing to do that?"))
+        yesno = gu.dialog_yesno(_("Automatic bug reports are often mostly useless because people omit their email address and add very little info about what happened. Fixing bugs is difficult if we cannot contact you and ask for more information.\n\nI would prefer if you open a web browser and report your bug to the bug tracker at http://bugs.solfege.org.\n\nThis will give your bug report higher priority and it will be fixed faster.\n\nAre you willing to do that?"))
         if yesno:
             return
         self.m_send_exception = 'Nothing'
@@ -90,7 +90,8 @@ class TracebackWindow(gtk.Dialog):
            (not self.m_show_gtk_warnings and (
             "GtkWarning" in txt
             or "PangoWarning" in txt
-            or "Python C API version mismatch for module _solfege_c_midi" in txt
+            or ("Python C API version mismatch" in txt and 
+                ("solfege_c_midi" in txt or "swig" in txt))
             )):
             print txt
             return
